@@ -97,7 +97,8 @@ init_purge_state(void)
 {
     purge_state.module = THE_NON_VALUE;
 
-    erts_smp_mtx_init(&purge_state.mtx, "purge_state");
+    erts_smp_mtx_init(&purge_state.mtx, "purge_state", NIL,
+        ERTS_LOCK_FLAGS_PROPERTY_STATIC | ERTS_LOCK_FLAGS_CATEGORY_GENERIC);
 
     purge_state.pending_purge_lambda =
 	erts_export_put(am_erts_code_purger, am_pending_purge_lambda, 3);
@@ -118,7 +119,9 @@ init_purge_state(void)
 void
 erts_beam_bif_load_init(void)
 {
-    erts_smp_mtx_init(&release_literal_areas.mtx, "release_literal_areas");
+    erts_smp_mtx_init(&release_literal_areas.mtx, "release_literal_areas", NIL,
+        ERTS_LOCK_FLAGS_PROPERTY_STATIC | ERTS_LOCK_FLAGS_CATEGORY_GENERIC);
+
     release_literal_areas.first = NULL;
     release_literal_areas.last = NULL;
     erts_smp_atomic_init_nob(&erts_copy_literal_area__,
@@ -258,7 +261,7 @@ struct m {
     Binary* code;
     Eterm module;
     Module* modp;
-    Uint exception;
+    Eterm exception;
 };
 
 static Eterm staging_epilogue(Process* c_p, int, Eterm res, int, struct m*, int, int);
@@ -279,7 +282,7 @@ exception_list(Process* p, Eterm tag, struct m* mp, Sint exceptions)
     Eterm res = NIL;
 
     while (exceptions > 0) {
-	if (mp->exception) {
+	if (is_value(mp->exception)) {
 	    res = CONS(hp, mp->module, res);
 	    hp += 2;
 	    exceptions--;
@@ -380,9 +383,9 @@ finish_loading_1(BIF_ALIST_1)
 
     exceptions = 0;
     for (i = 0; i < n; i++) {
-	p[i].exception = 0;
+	p[i].exception = THE_NON_VALUE;
 	if (p[i].modp->seen) {
-	    p[i].exception = 1;
+	    p[i].exception = am_duplicated;
 	    exceptions++;
 	}
 	p[i].modp->seen = 1;
@@ -416,9 +419,9 @@ finish_loading_1(BIF_ALIST_1)
 
     exceptions = 0;
     for (i = 0; i < n; i++) {
-	p[i].exception = 0;
+	p[i].exception = THE_NON_VALUE;
 	if (p[i].modp->curr.code_hdr && p[i].modp->old.code_hdr) {
-	    p[i].exception = 1;
+	    p[i].exception = am_not_purged;
 	    exceptions++;
 	}
     }
@@ -439,7 +442,7 @@ finish_loading_1(BIF_ALIST_1)
 	    retval = erts_finish_loading(p[i].code, BIF_P, 0, &mod);
 	    ASSERT(retval == NIL || retval == am_on_load);
 	    if (retval == am_on_load) {
-		p[i].exception = 1;
+		p[i].exception = am_on_load;
 		exceptions++;
 	    }
 	}
@@ -472,7 +475,8 @@ staging_epilogue(Process* c_p, int commit, Eterm res, int is_blocking,
 	    erts_commit_staging_code_ix();
 
 	    for (i=0; i < nmods; i++) {
-		if (mods[i].modp->curr.code_hdr) {
+		if (mods[i].modp->curr.code_hdr
+                    && mods[i].exception != am_on_load) {
 		    set_default_trace_pattern(mods[i].module);
 		}
 	      #ifdef HIPE
@@ -697,6 +701,7 @@ BIF_RETTYPE delete_module_1(BIF_ALIST_1)
 	Eterm retval;
 	mod.module = BIF_ARG_1;
 	mod.modp = modp;
+        mod.exception = THE_NON_VALUE;
 	retval = staging_epilogue(BIF_P, success, res, is_blocking, &mod, 1, 0);
 	return retval;
     }
