@@ -438,16 +438,20 @@ erts_sys_pre_init(void)
     /* After creation in parent */
     eid.thread_create_parent_func = thr_create_cleanup,
 
+#ifdef ERTS_ENABLE_LOCK_COUNT
+    erts_lcnt_pre_thr_init();
+#endif
+
     erts_thr_init(&eid);
 
+#ifdef ERTS_ENABLE_LOCK_COUNT
+    erts_lcnt_post_thr_init();
+#endif
+
 #ifdef ERTS_ENABLE_LOCK_CHECK
     erts_lc_init();
 #endif
 
-#ifdef ERTS_ENABLE_LOCK_COUNT
-    erts_lcnt_init();
-#endif
-
 #endif /* USE_THREADS */
 
     erts_init_sys_time_sup();
@@ -785,6 +789,8 @@ signalterm_to_signum(Eterm signal)
     }
 }
 
+#ifdef ERTS_SMP
+
 static ERTS_INLINE Eterm
 signum_to_signalterm(int signum)
 {
@@ -806,6 +812,8 @@ signum_to_signalterm(int signum)
     }
 }
 
+#endif
+
 #ifndef ERTS_SMP
 static ERTS_INLINE Uint
 signum_to_signalstate(int signum)
@@ -1545,7 +1553,8 @@ erl_sys_args(int* argc, char** argv)
 {
     int i, j;
 
-    erts_smp_rwmtx_init(&environ_rwmtx, "environ");
+    erts_smp_rwmtx_init(&environ_rwmtx, "environ", NIL,
+        ERTS_LOCK_FLAGS_PROPERTY_STATIC | ERTS_LOCK_FLAGS_CATEGORY_GENERIC);
 
     i = 1;
 
