@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2017. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -974,9 +974,6 @@ generate(_Fun, 0) ->
 generate(Fun, N) ->
     [Fun() | generate(Fun, N-1)].
 
-start_node(Config) ->
-    start_node(Config, "").
-
 start_node(Config, Args) ->
     TestCase = proplists:get_value(testcase, Config),
     PA = filename:dirname(code:which(?MODULE)),
