@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1998-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1998-2017. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -488,24 +488,40 @@ registered_process(Config) when is_list(Config) ->
 
 same_time_yielding(Config) when is_list(Config) ->
     Mem = mem(),
+    Ref = make_ref(),
     SchdlrsOnln = erlang:system_info(schedulers_online),
     Tmo = erlang:monotonic_time(millisecond) + 3000,
     Tmrs = lists:map(fun (I) ->
                              process_flag(scheduler, (I rem SchdlrsOnln) + 1),
-                             erlang:start_timer(Tmo, self(), hej, [{abs, true}])
+                             erlang:start_timer(Tmo, self(), Ref, [{abs, true}])
                      end,
                      lists:seq(1, (?TIMEOUT_YIELD_LIMIT*3+1)*SchdlrsOnln)),
     true = mem_larger_than(Mem),
-    lists:foreach(fun (Tmr) -> receive {timeout, Tmr, hej} -> ok end end, Tmrs),
+    receive_all_timeouts(length(Tmrs), Ref),
     Done = erlang:monotonic_time(millisecond),
     true = Done >= Tmo,
+    MsAfterTmo = Done - Tmo,
+    io:format("Done ~p ms after Tmo\n", [MsAfterTmo]),
     case erlang:system_info(build_type) of
-        opt -> true = Done < Tmo + 200;
-        _ -> true = Done < Tmo + 1000
+        opt ->
+            true = MsAfterTmo < 200;
+        _ ->
+            true = MsAfterTmo < 1000
     end,
     Mem = mem(),
     ok.
 
+%% Read out all timeouts in receive queue order. This is efficient
+%% even if there are very many messages.
+
+receive_all_timeouts(0, _Ref) ->
+    ok;
+receive_all_timeouts(N, Ref) ->
+    receive
+        {timeout, _Tmr, Ref} ->
+            receive_all_timeouts(N-1, Ref)
+    end.
+
 same_time_yielding_with_cancel(Config) when is_list(Config) ->
     same_time_yielding_with_cancel_test(false, false).
 
