@@ -48,13 +48,10 @@
  *
  *  HEART_BEATING
  *
- *  This program expects a heart beat messages. If it does not receive a 
- *  heart beat message from Erlang within heart_beat_timeout seconds, it 
- *  reboots the system. The variable heart_beat_timeout is exported (so
- *  that it can be set from the shell in VxWorks, as is the variable
- *  heart_beat_report_delay). When using Solaris, the system is rebooted
- *  by executing the command stored in the environment variable
- *  HEART_COMMAND.
+ *  This program expects a heart beat message. If it does not receive a
+ *  heart beat message from Erlang within heart_beat_timeout seconds, it
+ *  reboots the system. The system is rebooted by executing the command
+ *  stored in the environment variable HEART_COMMAND.
  *
  *  BLOCKING DESCRIPTORS
  *
@@ -149,27 +146,17 @@ struct msg {
 /*  Maybe interesting to change */
 
 /* Times in seconds */
-#define  HEART_BEAT_BOOT_DELAY       60  /* 1 minute */
 #define  SELECT_TIMEOUT               5  /* Every 5 seconds we reset the
 					    watchdog timer */
 
 /* heart_beat_timeout is the maximum gap in seconds between two
-   consecutive heart beat messages from Erlang, and HEART_BEAT_BOOT_DELAY
-   is the the extra delay that wd_keeper allows for, to give heart a
-   chance to reboot in the "normal" way before the hardware watchdog
-   enters the scene. heart_beat_report_delay is the time allowed for reporting
-   before rebooting under VxWorks. */
+   consecutive heart beat messages from Erlang. */
 
 int heart_beat_timeout = 60;
-int heart_beat_report_delay = 30;
-int heart_beat_boot_delay = HEART_BEAT_BOOT_DELAY;
 /* All current platforms have a process identifier that
    fits in an unsigned long and where 0 is an impossible or invalid value */
 unsigned long heart_beat_kill_pid = 0;
 
-#define VW_WD_TIMEOUT (heart_beat_timeout+heart_beat_report_delay+heart_beat_boot_delay)
-#define SOL_WD_TIMEOUT (heart_beat_timeout+heart_beat_boot_delay)
-
 /* reasons for reboot */
 #define  R_TIMEOUT          (1)
 #define  R_CLOSED           (2)
@@ -297,7 +284,6 @@ free_env_val(char *value)
 static void get_arguments(int argc, char** argv) {
     int i = 1;
     int h;
-    int w;
     unsigned long p;
 
     while (i < argc) {
@@ -313,15 +299,6 @@ static void get_arguments(int argc, char** argv) {
 			    i++;
 			}
 		break;
-	    case 'w':
-		if (strcmp(argv[i], "-wt") == 0)
-		    if (sscanf(argv[i+1],"%i",&w) ==1)
-			if ((w > 10) && (w <= 65535)) {
-			    heart_beat_boot_delay = w;
-			    fprintf(stderr,"heart_beat_boot_delay = %d\n",w);
-			    i++;
-			}
-		break;
 	    case 'p':
 		if (strcmp(argv[i], "-pid") == 0)
 		    if (sscanf(argv[i+1],"%lu",&p) ==1){
@@ -347,7 +324,7 @@ static void get_arguments(int argc, char** argv) {
 	}
 	i++;
     }
-    debugf("arguments -ht %d -wt %d -pid %lu\n",h,w,p);
+    debugf("arguments -ht %d -pid %lu\n",h,p);
 }
 
 int main(int argc, char **argv) {
@@ -674,11 +651,6 @@ void win_system(char *command)
  */
 static void 
 do_terminate(int erlin_fd, int reason) {
-  /*
-    When we get here, we have HEART_BEAT_BOOT_DELAY secs to finish
-    (plus heart_beat_report_delay if under VxWorks), so we don't need
-    to call wd_reset().
-    */
   int ret = 0, tmo=0;
   char *tmo_env;
 
