@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2017. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -136,7 +136,7 @@ unix_autoconf(XConf) ->
 	true ->
 	    OSXEnv = macosx_cflags(),
 	    UnQuotedEnv = assign_vars(unquote(Env++OSXEnv)),
-	    io:format("Running ~s~nEnv: ~p~n",
+	    io:format("Running ~ts~nEnv: ~p~n",
 		      [lists:flatten(Configure ++ Args),UnQuotedEnv]),
 	    Port = open_port({spawn, lists:flatten(["\"",Configure,"\"",Args])},
 			     [stream, eof, {env,UnQuotedEnv}]),
