@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2015-2016. All Rights Reserved.
+%% Copyright Ericsson AB 2015-2017. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -22,7 +22,7 @@
 -export([all/0,suite/0,groups/0,init_per_suite/1,end_per_suite/1,
 	 init_per_group/2,end_per_group/2,
 	 integers/1,coverage/1,booleans/1,setelement/1,cons/1,
-	 tuple/1]).
+	 tuple/1,record_float/1,binary_float/1,float_compare/1]).
 
 suite() -> [{ct_hooks,[ts_install_cth]}].
 
@@ -37,7 +37,10 @@ groups() ->
        booleans,
        setelement,
        cons,
-       tuple
+       tuple,
+       record_float,
+       binary_float,
+       float_compare
       ]}].
 
 init_per_suite(Config) ->
@@ -126,5 +129,48 @@ tuple(_Config) ->
 do_tuple() ->
     {0, _} = {necessary}.
 
+-record(x, {a}).
+
+record_float(_Config) ->
+    17.0 = record_float(#x{a={0}}, 1700),
+    23.0 = record_float(#x{a={0}}, 2300.0),
+    {'EXIT',{if_clause,_}} = (catch record_float(#x{a={1}}, 88)),
+    {'EXIT',{if_clause,_}} = (catch record_float(#x{a={}}, 88)),
+    {'EXIT',{if_clause,_}} = (catch record_float(#x{}, 88)),
+    ok.
+
+record_float(R, N0) ->
+    N = N0 / 100,
+    if element(1, R#x.a) =:= 0 ->
+            N
+    end.
+
+binary_float(_Config) ->
+    <<-1/float>> = binary_negate_float(<<1/float>>),
+    ok.
+
+binary_negate_float(<<Float/float>>) ->
+    <<-Float/float>>.
+
+float_compare(_Config) ->
+    false = do_float_compare(-42.0),
+    false = do_float_compare(-42),
+    false = do_float_compare(0),
+    false = do_float_compare(0.0),
+    true = do_float_compare(42),
+    true = do_float_compare(42.0),
+    ok.
+
+do_float_compare(X) ->
+    %% ERL-433: Used to fail before OTP 20. Was accidentally fixed
+    %% in OTP 20. Add a test case to ensure it stays fixed.
+
+    Y = X + 1.0,
+    case X > 0 of
+        T when (T =:= nil) or (T =:= false) -> T;
+        _T -> Y > 0
+    end.
+
+
 id(I) ->
     I.
