@@ -119,9 +119,19 @@ file_1(Config) when is_list(Config) ->
     true = exists(Target),
     passed = run(Target, test, []),
 
+    %% Test option 'deterministic' as a compiler attribute.
+    Det = deterministic_module,
+    {DetPath, DetTarget} = get_files(Config, Det, "det_target"),
+    {ok,Det,DetCode} = compile:file(DetPath, [binary]),
+    {module,Det} = code:load_binary(Det, "", DetCode),
+    [{version,_}] = Det:module_info(compile),
+    true = code:delete(Det),
+    false = code:purge(Det),
+
     %% Cleanup.
     ok = file:delete(Target),
     ok = file:del_dir(filename:dirname(Target)),
+    ok = file:del_dir(filename:dirname(DetTarget)),
 
     %% There should not be any messages in the messages.
     receive
@@ -398,6 +408,7 @@ do_file_listings(DataDir, PrivDir, [File|Files]) ->
     ok = file:delete(filename:join(Listings, File ++ ".core")),
     do_listing(Simple, TargetDir, to_core, ".core"),
     do_listing(Simple, TargetDir, to_kernel, ".kernel"),
+    do_listing(Simple, TargetDir, to_dis, ".dis"),
 
     %% Final clean up.
     lists:foreach(fun(F) -> ok = file:delete(F) end,
@@ -413,6 +424,7 @@ listings_big(Config) when is_list(Config) ->
     do_listing(Big, TargetDir, 'E'),
     do_listing(Big, TargetDir, 'P'),
     do_listing(Big, TargetDir, dkern, ".kernel"),
+    do_listing(Big, TargetDir, to_dis, ".dis"),
 
     TargetNoext = filename:rootname(Target, code:objfile_extension()),
     {ok,big} = compile:file(TargetNoext, [from_asm,{outdir,TargetDir}]),
