@@ -42,7 +42,8 @@
 %% Common Test interface functions -----------------------------------
 %%--------------------------------------------------------------------
 suite() ->
-    [{ct_hooks,[ts_install_cth]}
+    [{ct_hooks,[ts_install_cth]},
+     {timetrap,{seconds, 30}}
     ].
 
 all() ->
@@ -57,7 +58,7 @@ all() ->
 groups() ->
     [
      {http, [], real_requests()},
-     {sim_http, [], only_simulated()},
+     {sim_http, [], only_simulated() ++ [process_leak_on_keepalive]},
      {https, [], real_requests()},
      {sim_https, [], only_simulated()},
      {misc, [], misc()}
@@ -114,10 +115,10 @@ only_simulated() ->
      invalid_chunk_size,
      headers_dummy,
      headers_with_obs_fold,
+     headers_conflict_chunked_with_length,
      empty_response_header,
      remote_socket_close,
      remote_socket_close_async,
-     process_leak_on_keepalive,
      transfer_encoding,
      transfer_encoding_identity,
      redirect_loop,
@@ -129,7 +130,8 @@ only_simulated() ->
      port_in_host_header,
      redirect_port_in_host_header,
      relaxed,
-     multipart_chunks
+     multipart_chunks,
+     stream_fun_server_close
     ].
 
 misc() ->
@@ -142,7 +144,6 @@ misc() ->
 %%--------------------------------------------------------------------
 
 init_per_suite(Config) ->
-    ct:timetrap({seconds, 30}),
     PrivDir = proplists:get_value(priv_dir, Config),
     DataDir = proplists:get_value(data_dir, Config),
     inets_test_lib:start_apps([inets]),
@@ -169,7 +170,6 @@ init_per_group(Group, Config0) when Group =:= sim_https; Group =:= https->
     catch crypto:stop(),
     try crypto:start() of
         ok ->
-            ct:timetrap({seconds, 30}),
             start_apps(Group),
             do_init_per_group(Group, Config0)
     catch
@@ -746,7 +746,7 @@ empty_body() ->
 empty_body(Config) when is_list(Config) ->
     URL = url(group_name(Config), "/empty.html", Config),
     {ok, {{_,200,_}, [_ | _], []}} =
-	httpc:request(get, {URL, []}, [{timeout, 500}], []).
+	httpc:request(get, {URL, []}, [], []).
 
 %%-------------------------------------------------------------------------
 
@@ -978,7 +978,6 @@ headers_dummy(Config) when is_list(Config) ->
 		       {"If-Range", "Sat, 29 Oct 1994 19:43:31 GMT"},
 		       {"If-Match", "*"},
 		       {"Content-Type", "text/plain"},
-		       {"Content-Encoding", "chunked"},
 		       {"Content-Length", "6"},
 		       {"Content-Language", "en"},
 		       {"Content-Location", "http://www.foobar.se"},
@@ -1004,6 +1003,18 @@ headers_with_obs_fold(Config) when is_list(Config) ->
 
 %%-------------------------------------------------------------------------
 
+headers_conflict_chunked_with_length(doc) ->
+    ["Test the code for handling headers with both Transfer-Encoding"
+     "and Content-Length which must receive error in default (not relaxed) mode"
+     "and must receive successful response in relaxed mode"];
+headers_conflict_chunked_with_length(Config) when is_list(Config) ->
+    Request = {url(group_name(Config), "/headers_conflict_chunked_with_length.html", Config), []},
+    {error, {could_not_parse_as_http, _}} = httpc:request(get, Request, [{relaxed, false}], []),
+    {ok,{{_,200,_},_,_}} = httpc:request(get, Request, [{relaxed, true}], []),
+    ok.
+
+%%-------------------------------------------------------------------------
+
 invalid_headers(Config) ->
     Request  = {url(group_name(Config), "/dummy.html", Config), [{"cookie", undefined}]},
     {error, _} = httpc:request(get, Request, [], []).
@@ -1178,6 +1189,22 @@ wait_for_whole_response(Config) when is_list(Config) ->
      RespSeqNumServer ! shutdown,
      ReqSeqNumServer ! shutdown.
 
+%%--------------------------------------------------------------------
+stream_fun_server_close() ->
+    [{doc, "Test that an error msg is received when using a receiver fun as stream target"}].
+stream_fun_server_close(Config) when is_list(Config) ->
+    Request  = {url(group_name(Config), "/delay_close.html", Config), []},
+    Self = self(),
+    Fun = fun(X) -> Self ! X end,
+    {ok, RequestId} = httpc:request(get, Request, [], [{sync, false}, {receiver, Fun}]),
+    receive
+        {RequestId, {error, Reason}} ->
+            ct:pal("Close ~p", [Reason]),
+            ok
+    after 13000 ->
+            ct:fail(did_not_receive_close)
+    end. 
+                                             
 %%--------------------------------------------------------------------
 %% Internal Functions ------------------------------------------------
 %%--------------------------------------------------------------------
@@ -1853,7 +1880,6 @@ handle_uri(_,"/dummy_headers.html",_,_,Socket,_) ->
     %% user to evaluate. This is not a valid response
     %% it only tests that the header handling code works.
     Head = "HTTP/1.1 200 ok\r\n" ++
-	"Content-Length:32\r\n" ++
 	"Pragma:1#no-cache\r\n"  ++
 	"Via:1.0 fred, 1.1 nowhere.com (Apache/1.1)\r\n"  ++
 	"Warning:1#pseudonym foobar\r\n"  ++
@@ -1883,6 +1909,15 @@ handle_uri(_,"/obs_folded_headers.html",_,_,_,_) ->
     " b\r\n\r\n"
     "Hello";
 
+handle_uri(_,"/headers_conflict_chunked_with_length.html",_,_,Socket,_) ->
+    Head =  "HTTP/1.1 200 ok\r\n"
+        "Content-Length:32\r\n"
+	"Transfer-Encoding:Chunked\r\n\r\n",
+    send(Socket, Head),
+    send(Socket, http_chunk:encode("<HTML><BODY>fo")),
+    send(Socket, http_chunk:encode("obar</BODY></HTML>")),
+    http_chunk:encode_last();
+
 handle_uri(_,"/capital_transfer_encoding.html",_,_,Socket,_) ->
     Head =  "HTTP/1.1 200 ok\r\n" ++
 	"Transfer-Encoding:Chunked\r\n\r\n",
@@ -2030,6 +2065,9 @@ handle_uri(_,"/multipart_chunks.html",_,_,Socket,_) ->
     send(Socket, Head),
     send_multipart_chunks(Socket),
     http_chunk:encode_last();
+handle_uri(_,"/delay_close.html",_,_,Socket,_) ->
+    ct:sleep(10000),
+    close(Socket);
 handle_uri("HEAD",_,_,_,_,_) ->
     "HTTP/1.1 200 ok\r\n" ++
 	"Content-Length:0\r\n\r\n";
