@@ -21,7 +21,79 @@
 %%
 -module(mnesia_schema_recovery_test).
 -author('hakan@erix.ericsson.se').
--compile([export_all]).
+
+-export([init_per_testcase/2, end_per_testcase/2,
+         init_per_group/2, end_per_group/2,
+         all/0, groups/0]).
+
+-export([interrupted_before_create_ram/1,
+         interrupted_before_create_disc/1,
+         interrupted_before_create_do/1,
+         interrupted_before_create_nostore/1,
+         interrupted_before_delete_ram/1,
+         interrupted_before_delete_disc/1,
+         interrupted_before_delete_do/1,
+         interrupted_before_add_ram/1,
+         interrupted_before_add_disc/1,
+         interrupted_before_add_do/1,
+         interrupted_before_add_kill_copier/1,
+         interrupted_before_move_ram/1,
+         interrupted_before_move_disc/1,
+         interrupted_before_move_do/1,
+         interrupted_before_move_kill_copier/1,
+         interrupted_before_delcopy_ram/1,
+         interrupted_before_delcopy_disc/1,
+         interrupted_before_delcopy_do/1,
+         interrupted_before_delcopy_kill_copier/1,
+         interrupted_before_addindex_ram/1,
+         interrupted_before_addindex_disc/1,
+         interrupted_before_addindex_do/1,
+         interrupted_before_delindex_ram/1,
+         interrupted_before_delindex_disc/1,
+         interrupted_before_delindex_do/1,
+         interrupted_before_change_type_ram2disc/1,
+         interrupted_before_change_type_ram2do/1,
+         interrupted_before_change_type_disc2ram/1,
+         interrupted_before_change_type_disc2do/1,
+         interrupted_before_change_type_do2ram/1,
+         interrupted_before_change_type_do2disc/1,
+         interrupted_before_change_type_other_node/1,
+         interrupted_before_change_schema_type/1,
+         interrupted_after_create_ram/1,
+         interrupted_after_create_disc/1,
+         interrupted_after_create_do/1,
+         interrupted_after_create_nostore/1,
+         interrupted_after_delete_ram/1,
+         interrupted_after_delete_disc/1,
+         interrupted_after_delete_do/1,
+         interrupted_after_add_ram/1,
+         interrupted_after_add_disc/1,
+         interrupted_after_add_do/1,
+         interrupted_after_add_kill_copier/1,
+         interrupted_after_move_ram/1,
+         interrupted_after_move_disc/1,
+         interrupted_after_move_do/1,
+         interrupted_after_move_kill_copier/1,
+         interrupted_after_delcopy_ram/1,
+         interrupted_after_delcopy_disc/1,
+         interrupted_after_delcopy_do/1,
+         interrupted_after_delcopy_kill_copier/1,
+         interrupted_after_addindex_ram/1,
+         interrupted_after_addindex_disc/1,
+         interrupted_after_addindex_do/1,
+         interrupted_after_delindex_ram/1,
+         interrupted_after_delindex_disc/1,
+         interrupted_after_delindex_do/1,
+         interrupted_after_change_type_ram2disc/1,
+         interrupted_after_change_type_ram2do/1,
+         interrupted_after_change_type_disc2ram/1,
+         interrupted_after_change_type_disc2do/1,
+         interrupted_after_change_type_do2ram/1,
+         interrupted_after_change_type_do2disc/1,
+         interrupted_after_change_type_other_node/1,
+         interrupted_after_change_schema_type/1]).
+
+
 -include("mnesia_test_lib.hrl").
 
 init_per_testcase(Func, Conf) ->
