@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1996-2016. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2017. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -94,7 +94,11 @@ make_script(RelName, Output, Flags) when is_list(RelName),
 		    Warnings = wsasl(Flags, Warnings0),
 		    case systools_lib:werror(Flags, Warnings) of
 			true ->
-			    return(ok,Warnings,Flags);
+                            Warnings1 = [W || {warning,W}<-Warnings],
+			    return({error,?MODULE,
+                                    {warnings_treated_as_errors,Warnings1}},
+                                   Warnings,
+                                   Flags);
 			false ->
 			    case generate_script(Output,Release,Appls,Flags) of
 				ok ->
@@ -115,7 +119,6 @@ make_script(RelName, _Output, Flags) when is_list(Flags) ->
 make_script(RelName, _Output, Flags) ->
     badarg(Flags,[RelName, Flags]).
 
-
 wsasl(Options, Warnings) ->
     case lists:member(no_warn_sasl,Options) of
 	true -> lists:delete({warning,missing_sasl},Warnings);
@@ -148,21 +151,10 @@ get_outdir(Flags) ->
 return(ok,Warnings,Flags) ->
     case member(silent,Flags) of
 	true ->
-	    case systools_lib:werror(Flags, Warnings) of
-		true ->
-		    error;
-		false ->
-		    {ok,?MODULE,Warnings}
-	    end;
+            {ok,?MODULE,Warnings};
 	_ ->
-	    case member(warnings_as_errors,Flags) of
-		true ->
-		    io:format("~ts",[format_warning(Warnings, true)]),
-		    error;
-		false ->
-		    io:format("~ts",[format_warning(Warnings)]),
-		    ok
-	    end
+            io:format("~ts",[format_warning(Warnings)]),
+            ok
     end;
 return({error,Mod,Error},_,Flags) ->
     case member(silent,Flags) of
@@ -300,6 +292,8 @@ add_apply_upgrade(Script,Args) ->
 %%              {variables,[{Name,AbsString}]}
 %%              {machine, jam | beam | vee}
 %%              {var_tar, include | ownfile | omit}
+%%              no_warn_sasl
+%%              warnings_as_errors
 %%
 %% The tar file contains:
 %%         lib/App-Vsn/ebin
@@ -332,13 +326,23 @@ make_tar(RelName, Flags) when is_list(RelName), is_list(Flags) ->
 	    Path  = make_set(Path1 ++ code:get_path()),
 	    ModTestP = {member(src_tests, Flags),xref_p(Flags)},
 	    case get_release(RelName, Path, ModTestP, machine(Flags)) of
-		{ok, Release, Appls, Warnings} ->
-		    case catch mk_tar(RelName, Release, Appls, Flags, Path1) of
-			ok ->
-			    return(ok,Warnings,Flags);
-			Error ->
-			    return(Error,Warnings,Flags)
-		    end;
+		{ok, Release, Appls, Warnings0} ->
+		    Warnings = wsasl(Flags, Warnings0),
+		    case systools_lib:werror(Flags, Warnings) of
+			true ->
+                            Warnings1 = [W || {warning,W}<-Warnings],
+			    return({error,?MODULE,
+                                    {warnings_treated_as_errors,Warnings1}},
+                                   Warnings,
+                                   Flags);
+			false ->
+                            case catch mk_tar(RelName, Release, Appls, Flags, Path1) of
+                                ok ->
+                                    return(ok,Warnings,Flags);
+                                Error ->
+                                    return(Error,Warnings,Flags)
+                            end
+                    end;
 		Error ->
 		    return(Error,[],Flags)
 	    end;
@@ -643,6 +647,8 @@ get_items([], _Dict) ->
 
 check_item({_,{mod,{M,A}}},_) when is_atom(M) ->
     {M,A};
+check_item({_,{mod,[]}},_) -> % default mod is [], so accept as entry
+    [];
 check_item({_,{vsn,Vsn}},I) ->
     case string_p(Vsn) of
 	true -> Vsn;
@@ -678,6 +684,8 @@ check_item({_,{modules,Mods}},I) ->
 	true -> Mods;
 	_ -> throw({bad_param, I})
     end;
+check_item({_,{start_phases,undefined}},_) -> % default start_phase is undefined,
+    undefined;                                % so accept as entry
 check_item({_,{start_phases,Phase}},I) ->
     case t_list_p(Phase) of
 	true -> Phase;
@@ -1144,10 +1152,10 @@ generate_script(Output, Release, Appls, Flags) ->
 	     },
 
     ScriptFile = Output ++ ".script",
-    case file:open(ScriptFile, [write]) of
+    case file:open(ScriptFile, [write,{encoding,utf8}]) of
 	{ok, Fd} ->
-	    io:format(Fd, "%% script generated at ~w ~w\n~p.\n",
-		      [date(), time(), Script]),
+	    io:format(Fd, "%% ~s\n%% script generated at ~w ~w\n~tp.\n",
+		      [epp:encoding_to_string(utf8), date(), time(), Script]),
 	    case file:close(Fd) of
 		ok ->
 		    BootFile = Output ++ ".boot",
@@ -1900,8 +1908,10 @@ del_tar(Tar, TarName) ->
     file:delete(TarName).
 
 add_to_tar(Tar, FromFile, ToFile) ->
-    case erl_tar:add(Tar, FromFile, ToFile, [compressed, dereference]) of
+    case catch erl_tar:add(Tar, FromFile, ToFile, [compressed, dereference]) of
 	ok -> ok;
+        {'EXIT', Reason} ->
+            throw({error, {tar_error, {add, FromFile, Reason}}});
 	{error, Error} ->
 	    throw({error, {tar_error, {add, FromFile, Error}}})
     end.
@@ -2109,90 +2119,80 @@ cas([Y | Args], X) ->
 
 %% Check Options for make_tar
 check_args_tar(Args) ->
-    cat(Args, {undef, undef, undef, undef, undef, undef, undef, undef, undef, undef, []}). 
+    cat(Args, []).
 
-cat([], {_Path,_Sil,_Dirs,_Erts,_Test,_Var,_VarTar,_Mach,_Xref,_XrefApps, X}) ->
+cat([], X) ->
     X;
 %%% path ---------------------------------------------------------------
-cat([{path, P} | Args], {Path, Sil, Dirs, Erts, Test, 
-			 Var, VarTar, Mach, Xref, XrefApps, X}) when is_list(P) -> 
+cat([{path, P} | Args], X) when is_list(P) ->
     case check_path(P) of
 	ok ->
-	    cat(Args, {P, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X});
+	    cat(Args, X);
 	error ->
-	    cat(Args, {Path, Sil, Dirs, Erts, Test, 
-		       Var, VarTar, Mach, Xref, XrefApps, X++[{path,P}]})
+	    cat(Args, X++[{path,P}])
     end;
 %%% silent -------------------------------------------------------------
-cat([silent | Args], {Path, _Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X}) ->
-    cat(Args, {Path, silent, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X});
+cat([silent | Args], X) ->
+    cat(Args, X);
 %%% dirs ---------------------------------------------------------------
-cat([{dirs, D} | Args], {Path, Sil, Dirs, Erts, Test, 
-			    Var, VarTar, Mach, Xref, XrefApps, X}) ->
+cat([{dirs, D} | Args], X) ->
     case check_dirs(D) of
 	ok ->
-	    cat(Args, {Path, Sil, D, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X});
+	    cat(Args, X);
 	error ->
-	    cat(Args, {Path, Sil, Dirs, Erts, Test, 
-		       Var, VarTar, Mach, Xref, XrefApps, X++[{dirs, D}]})
+	    cat(Args, X++[{dirs, D}])
     end;
 %%% erts ---------------------------------------------------------------
-cat([{erts, E} | Args], {Path, Sil, Dirs, _Erts, Test, 
-			 Var, VarTar, Mach, Xref, XrefApps, X}) when is_list(E)->
-    cat(Args, {Path, Sil, Dirs, E, Test, Var, VarTar, Mach, Xref, XrefApps, X});
+cat([{erts, E} | Args], X) when is_list(E)->
+    cat(Args, X);
 %%% src_tests ----------------------------------------------------
-cat([src_tests | Args], {Path, Sil, Dirs, Erts, _Test, Var, VarTar, Mach, Xref, XrefApps, X}) ->
-    cat(Args, {Path, Sil, Dirs, Erts, src_tests, Var, VarTar, Mach, 
-	       Xref, XrefApps, X});
+cat([src_tests | Args], X) ->
+    cat(Args, X);
 %%% variables ----------------------------------------------------------
-cat([{variables, V} | Args], {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X}) when is_list(V) ->
+cat([{variables, V} | Args], X) when is_list(V) ->
     case check_vars(V) of
 	ok ->
-	    cat(Args, {Path, Sil, Dirs, Erts, Test, V, VarTar, Mach, Xref, XrefApps, X});
+	    cat(Args, X);
 	error ->
-	    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, 
-			     Xref, XrefApps, X++[{variables, V}]})
+	    cat(Args, X++[{variables, V}])
     end;
 %%% var_tar ------------------------------------------------------------
-cat([{var_tar, VT} | Args], {Path, Sil, Dirs, Erts, Test, 
-			    Var, _VarTar, Mach, Xref, XrefApps, X}) when VT == include ->
-    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, include, Mach, Xref, XrefApps, X});
-cat([{var_tar, VT} | Args], {Path, Sil, Dirs, Erts, Test, 
-			    Var, _VarTar, Mach, Xref, XrefApps, X}) when VT == ownfile ->
-    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, ownfile, Mach, Xref, XrefApps, X});
-cat([{var_tar, VT} | Args], {Path, Sil, Dirs, Erts, Test, 
-			    Var, _VarTar, Mach, Xref, XrefApps, X}) when VT == omit ->
-    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, omit, Mach, Xref, XrefApps, X});
+cat([{var_tar, VT} | Args], X) when VT == include;
+                                    VT == ownfile;
+                                    VT == omit ->
+    cat(Args, X);
 %%% machine ------------------------------------------------------------
-cat([{machine, M} | Args], {Path, Sil, Dirs, Erts, Test, 
-			    Var, VarTar, Mach, Xref, XrefApps, X}) when is_atom(M) ->
-    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X});
+cat([{machine, M} | Args], X) when is_atom(M) ->
+    cat(Args, X);
 %%% exref --------------------------------------------------------------
-cat([exref | Args], {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, _Xref, XrefApps, X})  ->
-    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, exref, XrefApps, X});
+cat([exref | Args], X)  ->
+    cat(Args, X);
 %%% exref Apps ---------------------------------------------------------
-cat([{exref, Apps} | Args], {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X}) when is_list(Apps) ->
+cat([{exref, Apps} | Args], X) when is_list(Apps) ->
     case check_apps(Apps) of 
 	ok ->
-	    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, 
-			     Xref, Apps, X});
+	    cat(Args, X);
 	error ->
-	    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, 
-			     Xref, XrefApps, X++[{exref, Apps}]})
+	    cat(Args, X++[{exref, Apps}])
     end;
 %%% outdir Dir ---------------------------------------------------------
-cat([{outdir, Dir} | Args], {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X}) when is_list(Dir) ->
-    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach,
-	       Xref, XrefApps, X});
+cat([{outdir, Dir} | Args], X) when is_list(Dir) ->
+    cat(Args, X);
 %%% otp_build (secret, not documented) ---------------------------------
-cat([otp_build | Args], {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X})  ->
-    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X});
+cat([otp_build | Args], X)  ->
+    cat(Args, X);
+%%% warnings_as_errors ----
+cat([warnings_as_errors | Args], X) ->
+    cat(Args, X);
+%%% no_warn_sasl ----
+cat([no_warn_sasl | Args], X) ->
+    cat(Args, X);
 %%% no_module_tests (kept for backwards compatibility, but ignored) ----
-cat([no_module_tests | Args], {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X}) ->
-    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X});
+cat([no_module_tests | Args], X) ->
+    cat(Args, X);
 %%% ERROR --------------------------------------------------------------
-cat([Y | Args], {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X}) ->
-    cat(Args, {Path, Sil, Dirs, Erts, Test, Var, VarTar, Mach, Xref, XrefApps, X++[Y]}).
+cat([Y | Args], X) ->
+    cat(Args, X++[Y]).
 
 check_path([]) ->
     ok;
@@ -2234,9 +2234,9 @@ check_apps(_) ->
 format_error(badly_formatted_release) ->
     io_lib:format("Syntax error in the release file~n",[]);
 format_error({illegal_name, Name}) ->
-    io_lib:format("Illegal name (~p) in the release file~n",[Name]);
+    io_lib:format("Illegal name (~tp) in the release file~n",[Name]);
 format_error({illegal_form, Form}) ->
-    io_lib:format("Illegal tag in the release file: ~p~n",[Form]);
+    io_lib:format("Illegal tag in the release file: ~tp~n",[Form]);
 format_error({missing_parameter,Par}) ->
     io_lib:format("Missing parameter (~p) in the release file~n",[Par]);
 format_error({illegal_applications,Names}) ->
@@ -2251,7 +2251,7 @@ format_error({mandatory_app,Name,Type}) ->
 format_error({duplicate_register,Dups}) ->
     io_lib:format("Duplicated register names: ~n~ts",
 		  [map(fun({{Reg,App1,_,_},{Reg,App2,_,_}}) ->
-			       io_lib:format("\t~w registered in ~w and ~w~n",
+			       io_lib:format("\t~tw registered in ~w and ~w~n",
 					     [Reg,App1,App2])
 		       end, Dups)]);
 format_error({undefined_applications,Apps}) ->
@@ -2275,26 +2275,29 @@ format_error({modules,ModErrs}) ->
 format_error({circular_dependencies,Apps}) ->
     io_lib:format("Circular dependencies among applications: ~p~n",[Apps]);
 format_error({not_found,File}) ->
-    io_lib:format("File not found: ~p~n",[File]);
+    io_lib:format("File not found: ~tp~n",[File]);
 format_error({parse,File,{Line,Mod,What}}) ->
     Str = Mod:format_error(What),
     io_lib:format("~ts:~w: ~ts\n",[File, Line, Str]);
 format_error({read,File}) ->
-    io_lib:format("Cannot read ~p~n",[File]);
+    io_lib:format("Cannot read ~tp~n",[File]);
 format_error({open,File,Error}) ->
-    io_lib:format("Cannot open ~p - ~ts~n",
+    io_lib:format("Cannot open ~tp - ~ts~n",
 		  [File,file:format_error(Error)]);
 format_error({close,File,Error}) ->
-    io_lib:format("Cannot close ~p - ~ts~n",
+    io_lib:format("Cannot close ~tp - ~ts~n",
 		  [File,file:format_error(Error)]);
 format_error({delete,File,Error}) ->
-    io_lib:format("Cannot delete ~p - ~ts~n",
+    io_lib:format("Cannot delete ~tp - ~ts~n",
 		  [File,file:format_error(Error)]);
 format_error({tar_error,What}) ->
     form_tar_err(What);
+format_error({warnings_treated_as_errors,Warnings}) ->
+    io_lib:format("Warnings being treated as errors:~n~ts",
+                  [map(fun(W) -> form_warn("",W) end, Warnings)]);
 format_error(ListOfErrors) when is_list(ListOfErrors) ->
     format_errors(ListOfErrors);
-format_error(E) -> io_lib:format("~p~n",[E]).
+format_error(E) -> io_lib:format("~tp~n",[E]).
 
 format_errors(ListOfErrors) ->
     map(fun({error,E}) -> form_err(E);
@@ -2310,19 +2313,19 @@ form_err({module_not_found,App,Mod}) ->
 form_err({error_add_appl, {Name, {tar_error, What}}}) ->
     io_lib:format("~p: ~ts~n",[Name,form_tar_err(What)]);
 form_err(E) ->
-    io_lib:format("~p~n",[E]).
+    io_lib:format("~tp~n",[E]).
 
 form_reading({not_found,File}) ->
-    io_lib:format("File not found: ~p~n",[File]);
+    io_lib:format("File not found: ~tp~n",[File]);
 form_reading({application_vsn, {Name,Vsn}}) ->
-    io_lib:format("Application ~ts with version ~p not found~n",[Name, Vsn]);
+    io_lib:format("Application ~ts with version ~tp not found~n",[Name, Vsn]);
 form_reading({parse,File,{Line,Mod,What}}) ->
     Str = Mod:format_error(What),
     io_lib:format("~ts:~w: ~ts\n",[File, Line, Str]);
 form_reading({read,File}) ->
-    io_lib:format("Cannot read ~p~n",[File]);
+    io_lib:format("Cannot read ~tp~n",[File]);
 form_reading({{bad_param, P},_}) ->
-    io_lib:format("Bad parameter in .app file: ~p~n",[P]);
+    io_lib:format("Bad parameter in .app file: ~tp~n",[P]);
 form_reading({{missing_param,P},_}) ->
     io_lib:format("Missing parameter in .app file: ~p~n",[P]);
 form_reading({badly_formatted_application,_}) ->
@@ -2331,12 +2334,12 @@ form_reading({override_include,Apps}) ->
     io_lib:format("Tried to include not (in .app file) specified applications: ~p~n",
 		  [Apps]);
 form_reading({no_valid_version, {{_, SVsn}, {_, File, FVsn}}}) ->
-    io_lib:format("No valid version (~p) of .app file found. Found file ~p with version ~p~n", 
+    io_lib:format("No valid version (~tp) of .app file found. Found file ~tp with version ~tp~n", 
 		  [SVsn, File, FVsn]);
 form_reading({parse_error, {File, Line, Error}}) ->
-    io_lib:format("Parse error in file: ~p.  Line: ~w  Error: ~p; ~n", [File, Line, Error]);
+    io_lib:format("Parse error in file: ~tp.  Line: ~w  Error: ~tp; ~n", [File, Line, Error]);
 form_reading(W) ->
-    io_lib:format("~p~n",[W]).
+    io_lib:format("~tp~n",[W]).
 
 form_tar_err({open, File, Error}) ->
     io_lib:format("Cannot open tar file ~ts - ~ts~n",
@@ -2348,35 +2351,26 @@ form_tar_err({add, File, Error}) ->
 %% Format warning
 
 format_warning(Warnings) ->
-    format_warning(Warnings, false).
+    map(fun({warning,W}) -> form_warn("*WARNING* ", W) end, Warnings).
 
-format_warning(Warnings, Werror) ->
-    Prefix = case Werror of
-		 true ->
-		     "";
-		 false ->
-		     "*WARNING* "
-	     end,
-    map(fun({warning,W}) -> form_warn(Prefix, W) end, Warnings).
-
-form_warn(Prefix, {source_not_found,{Mod,_,App,_,_}}) ->
+form_warn(Prefix, {source_not_found,{Mod,App,_}}) ->
     io_lib:format("~ts~w: Source code not found: ~w.erl~n",
 		  [Prefix,App,Mod]);
 form_warn(Prefix, {{parse_error, File},{_,_,App,_,_}}) ->
-    io_lib:format("~ts~w: Parse error: ~p~n",
+    io_lib:format("~ts~w: Parse error: ~tp~n",
 		  [Prefix,App,File]);
-form_warn(Prefix, {obj_out_of_date,{Mod,_,App,_,_}}) ->
+form_warn(Prefix, {obj_out_of_date,{Mod,App,_}}) ->
     io_lib:format("~ts~w: Object code (~w) out of date~n",
 		  [Prefix,App,Mod]);
 form_warn(Prefix, {exref_undef, Undef}) ->
     F = fun({M,F,A}) ->
-		io_lib:format("~tsUndefined function ~w:~w/~w~n",
+		io_lib:format("~tsUndefined function ~w:~tw/~w~n",
 			      [Prefix,M,F,A])
 	end,
     map(F, Undef);
 form_warn(Prefix, missing_sasl) ->
-    io_lib:format("~ts: Missing application sasl. "
+    io_lib:format("~tsMissing application sasl. "
 		  "Can not upgrade with this release~n",
 		  [Prefix]);
 form_warn(Prefix, What) ->
-    io_lib:format("~ts ~p~n", [Prefix,What]).
+    io_lib:format("~ts~tp~n", [Prefix,What]).
