@@ -46,6 +46,7 @@
 	 exec_key_differs2/1,
 	 exec_key_differs3/1,
 	 exec_key_differs_fail/1,
+         fail_daemon_start/1,
 	 idle_time_client/1,
 	 idle_time_server/1,
 	 inet6_option/1,
@@ -99,9 +100,13 @@ all() ->
      {group, ecdsa_sha2_nistp521_key},
      {group, dsa_pass_key},
      {group, rsa_pass_key},
+     {group, ecdsa_sha2_nistp256_pass_key},
+     {group, ecdsa_sha2_nistp384_pass_key},
+     {group, ecdsa_sha2_nistp521_pass_key},
      {group, host_user_key_differs},
      {group, key_cb},
      {group, internal_error},
+     {group, rsa_host_key_is_actualy_ecdsa},
      daemon_already_started,
      double_close,
      daemon_opt_fd,
@@ -118,12 +123,16 @@ groups() ->
      {ecdsa_sha2_nistp256_key, [], basic_tests()},
      {ecdsa_sha2_nistp384_key, [], basic_tests()},
      {ecdsa_sha2_nistp521_key, [], basic_tests()},
+     {rsa_host_key_is_actualy_ecdsa, [], [fail_daemon_start]},
      {host_user_key_differs, [], [exec_key_differs1,
 				  exec_key_differs2,
 				  exec_key_differs3,
 				  exec_key_differs_fail]},
      {dsa_pass_key, [], [pass_phrase]},
      {rsa_pass_key, [], [pass_phrase]},
+     {ecdsa_sha2_nistp256_pass_key, [], [pass_phrase]},
+     {ecdsa_sha2_nistp384_pass_key, [], [pass_phrase]},
+     {ecdsa_sha2_nistp521_pass_key, [], [pass_phrase]},
      {key_cb, [], [key_callback, key_callback_options]},
      {internal_error, [], [internal_error]},
      {login_bad_pwd_no_retry, [], [login_bad_pwd_no_retry1,
@@ -174,6 +183,31 @@ init_per_group(rsa_key, Config) ->
 	false ->
 	    {skip, unsupported_pub_key}
     end;
+init_per_group(rsa_host_key_is_actualy_ecdsa, Config) ->
+    case 
+        lists:member('ssh-rsa',
+		      ssh_transport:default_algorithms(public_key)) and
+        lists:member('ecdsa-sha2-nistp256',
+                     ssh_transport:default_algorithms(public_key))
+    of
+	true ->
+            DataDir = proplists:get_value(data_dir, Config),
+            PrivDir = proplists:get_value(priv_dir, Config),
+	    ssh_test_lib:setup_ecdsa("256", DataDir, PrivDir),
+            %% The following sets up bad rsa keys:
+            begin
+                UserDir = PrivDir,
+                System = filename:join(UserDir, "system"),
+                file:copy(filename:join(DataDir, "id_rsa"), filename:join(UserDir, "id_rsa")),
+                file:rename(filename:join(System, "ssh_host_ecdsa_key"), filename:join(System, "ssh_host_rsa_key")),
+                file:rename(filename:join(System, "ssh_host_ecdsa_key.pub"), filename:join(System, "ssh_host_rsa_key.pub")),
+                ssh_test_lib:setup_rsa_known_host(DataDir, UserDir),
+                ssh_test_lib:setup_rsa_auth_keys(DataDir, UserDir)
+            end,
+            Config;
+	false ->
+	    {skip, unsupported_pub_key}
+    end;
 init_per_group(ecdsa_sha2_nistp256_key, Config) ->
     case lists:member('ecdsa-sha2-nistp256',
 		      ssh_transport:default_algorithms(public_key)) of
@@ -229,6 +263,45 @@ init_per_group(dsa_pass_key, Config) ->
 	false ->
 	    {skip, unsupported_pub_key}
     end;
+init_per_group(ecdsa_sha2_nistp256_pass_key, Config) ->
+    DataDir = proplists:get_value(data_dir, Config),
+    PrivDir = proplists:get_value(priv_dir, Config),
+    case lists:member('ecdsa-sha2-nistp256',
+		      ssh_transport:default_algorithms(public_key))
+        andalso
+        ssh_test_lib:setup_ecdsa_pass_phrase("256", DataDir, PrivDir, "Password")
+    of
+	true ->
+	    [{pass_phrase, {ecdsa_pass_phrase, "Password"}}| Config];
+	false ->
+	    {skip, unsupported_pub_key}
+    end;
+init_per_group(ecdsa_sha2_nistp384_pass_key, Config) ->
+    DataDir = proplists:get_value(data_dir, Config),
+    PrivDir = proplists:get_value(priv_dir, Config),
+    case lists:member('ecdsa-sha2-nistp384',
+		      ssh_transport:default_algorithms(public_key))
+        andalso
+        ssh_test_lib:setup_ecdsa_pass_phrase("384", DataDir, PrivDir, "Password")
+    of
+	true ->
+	    [{pass_phrase, {ecdsa_pass_phrase, "Password"}}| Config];
+	false ->
+	    {skip, unsupported_pub_key}
+    end;
+init_per_group(ecdsa_sha2_nistp521_pass_key, Config) ->
+    DataDir = proplists:get_value(data_dir, Config),
+    PrivDir = proplists:get_value(priv_dir, Config),
+    case lists:member('ecdsa-sha2-nistp521',
+		      ssh_transport:default_algorithms(public_key))
+        andalso
+        ssh_test_lib:setup_ecdsa_pass_phrase("521", DataDir, PrivDir, "Password")
+    of
+	true ->
+	    [{pass_phrase, {ecdsa_pass_phrase, "Password"}}| Config];
+	false ->
+	    {skip, unsupported_pub_key}
+    end;
 init_per_group(host_user_key_differs, Config) ->
     Data = proplists:get_value(data_dir, Config),
     Sys = filename:join(proplists:get_value(priv_dir, Config), system_rsa),
@@ -241,7 +314,7 @@ init_per_group(host_user_key_differs, Config) ->
     file:copy(filename:join(Data, "ssh_host_rsa_key.pub"), filename:join(Sys, "ssh_host_rsa_key.pub")),
     file:copy(filename:join(Data, "id_ecdsa256"),         filename:join(Usr, "id_ecdsa")),
     file:copy(filename:join(Data, "id_ecdsa256.pub"),     filename:join(Usr, "id_ecdsa.pub")),
-    ssh_test_lib:setup_ecdsa_auth_keys("256", Usr, SysUsr),
+    ssh_test_lib:setup_ecdsa_auth_keys("256", Data, SysUsr),
     ssh_test_lib:setup_rsa_known_host(Sys, Usr),
     Config;
 init_per_group(key_cb, Config) ->
@@ -259,7 +332,8 @@ init_per_group(internal_error, Config) ->
     DataDir = proplists:get_value(data_dir, Config),
     PrivDir = proplists:get_value(priv_dir, Config),
     ssh_test_lib:setup_dsa(DataDir, PrivDir),
-    file:delete(filename:join(PrivDir, "system/ssh_host_dsa_key")),
+    %% In the test case the key will be deleted after the daemon start:
+    %% ... file:delete(filename:join(PrivDir, "system/ssh_host_dsa_key")),
     Config;
 init_per_group(dir_options, Config) ->
     PrivDir = proplists:get_value(priv_dir, Config),
@@ -306,6 +380,7 @@ init_per_group(dir_options, Config) ->
 init_per_group(_, Config) ->
     Config.
 
+
 end_per_group(dsa_key, Config) ->
     PrivDir = proplists:get_value(priv_dir, Config),
     ssh_test_lib:clean_dsa(PrivDir),
@@ -822,12 +897,17 @@ key_callback_options(Config) when is_list(Config) ->
 %%% Test that client does not hang if disconnects due to internal error
 internal_error(Config) when is_list(Config) ->
     process_flag(trap_exit, true),
-    SystemDir = filename:join(proplists:get_value(priv_dir, Config), system),
+    PrivDir = proplists:get_value(priv_dir, Config),
     UserDir = proplists:get_value(priv_dir, Config),
+    SystemDir = filename:join(PrivDir, system),
     
     {Pid, Host, Port} = ssh_test_lib:daemon([{system_dir, SystemDir},
                                              {user_dir, UserDir},
                                              {failfun, fun ssh_test_lib:failfun/2}]),
+
+    %% Now provoke an error in the following connect:
+    file:delete(filename:join(PrivDir, "system/ssh_host_dsa_key")), 
+
     {error, Error} =
         ssh:connect(Host, Port, [{silently_accept_hosts, true},
                                  {user_dir, UserDir},
@@ -855,6 +935,17 @@ send(Config) when is_list(Config) ->
     ssh:stop_daemon(Pid).
 
 
+%%--------------------------------------------------------------------
+%%%
+fail_daemon_start(Config) when is_list(Config) ->
+    process_flag(trap_exit, true),
+    SystemDir = filename:join(proplists:get_value(priv_dir, Config), system),
+    UserDir = proplists:get_value(priv_dir, Config),
+
+    {error,_} = ssh_test_lib:daemon([{system_dir, SystemDir},
+                                     {user_dir, UserDir},
+                                     {failfun, fun ssh_test_lib:failfun/2}]).
+
 %%--------------------------------------------------------------------
 %%% Test ssh:connection_info([peername, sockname])
 peername_sockname(Config) when is_list(Config) ->
@@ -1254,14 +1345,11 @@ shell_exit_status(Config) when is_list(Config) ->
 %%--------------------------------------------------------------------
 %% Due to timing the error message may or may not be delivered to
 %% the "tcp-application" before the socket closed message is recived
-check_error("Invalid state") ->
-    ok;
-check_error("Connection closed") ->
-    ok;
-check_error("Selection of key exchange algorithm failed"++_) ->
-    ok;
-check_error(Error) ->
-    ct:fail(Error).
+check_error("Invalid state") -> ok;
+check_error("Connection closed") -> ok;
+check_error("Selection of key exchange algorithm failed"++_) -> ok;
+check_error("No host key available") -> ok;
+check_error(Error) -> ct:fail(Error).
 
 basic_test(Config) ->
     ClientOpts = proplists:get_value(client_opts, Config),
