@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2016. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2017. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -36,7 +36,7 @@
 %%--------------------------------------------------------------------
 
 suite() ->
-    [{timetrap,{seconds,20}}].
+    [{timetrap,{seconds,60}}].
 
 all() -> 
     case os:find_executable("ssh") of
@@ -107,6 +107,9 @@ init_per_testcase(erlang_server_openssh_client_public_key_rsa, Config) ->
     chk_key(sshc, 'ssh-rsa', ".ssh/id_rsa", Config);
 init_per_testcase(erlang_client_openssh_server_publickey_dsa, Config) ->
     chk_key(sshd, 'ssh-dss', ".ssh/id_dsa", Config);
+init_per_testcase(erlang_client_openssh_server_publickey_rsa, Config) ->
+    chk_key(sshd, 'ssh-rsa', ".ssh/id_rsa", Config);
+
 init_per_testcase(erlang_server_openssh_client_renegotiate, Config) ->
     case os:type() of
 	{unix,_} -> ssh:start(), Config;
@@ -153,7 +156,7 @@ erlang_shell_client_openssh_server(Config) when is_list(Config) ->
     IO = ssh_test_lib:start_io_server(),
     Shell = ssh_test_lib:start_shell(?SSH_DEFAULT_PORT, IO),
     IO ! {input, self(), "echo Hej\n"},
-    receive_data("Hej"),
+    receive_data("Hej", undefined),
     IO ! {input, self(), "exit\n"},
     receive_logout(),
     receive_normal_exit(Shell).
@@ -322,73 +325,51 @@ erlang_client_openssh_server_setenv(Config) when is_list(Config) ->
 %% setenv not meaningfull on erlang ssh daemon!
 
 %%--------------------------------------------------------------------
-erlang_client_openssh_server_publickey_rsa() ->
-    [{doc, "Validate using rsa publickey."}].
-erlang_client_openssh_server_publickey_rsa(Config) when is_list(Config) ->
-    {ok,[[Home]]} = init:get_argument(home),
-    KeyFile =  filename:join(Home, ".ssh/id_rsa"),
-    case file:read_file(KeyFile) of
-	{ok, Pem} ->
-	    case public_key:pem_decode(Pem) of
-		[{_,_, not_encrypted}] ->
-		    ConnectionRef =
-			ssh_test_lib:connect(?SSH_DEFAULT_PORT,
-					     [{public_key_alg, ssh_rsa},
-					      {user_interaction, false},
-					      silently_accept_hosts]),
-		    {ok, Channel} =
-			ssh_connection:session_channel(ConnectionRef, infinity),
-		    ok = ssh_connection:close(ConnectionRef, Channel),
-		    ok = ssh:close(ConnectionRef);
-		_ ->
-		    {skip, {error, "Has pass phrase can not be used by automated test case"}} 
-	    end;
-	_ ->
-	    {skip, "no ~/.ssh/id_rsa"}  
-    end.
-	
+erlang_client_openssh_server_publickey_rsa(Config) ->
+    erlang_client_openssh_server_publickey_X(Config, 'ssh-rsa').
+    
+erlang_client_openssh_server_publickey_dsa(Config) ->
+    erlang_client_openssh_server_publickey_X(Config, 'ssh-dss').
 
-%%--------------------------------------------------------------------
-erlang_client_openssh_server_publickey_dsa() ->
-    [{doc, "Validate using dsa publickey."}].
-erlang_client_openssh_server_publickey_dsa(Config) when is_list(Config) ->
+
+erlang_client_openssh_server_publickey_X(_Config, Alg) ->
     ConnectionRef =
-	ssh_test_lib:connect(?SSH_DEFAULT_PORT,
-			     [{public_key_alg, ssh_dsa},
-			      {user_interaction, false},
-			      silently_accept_hosts]),
+        ssh_test_lib:connect(?SSH_DEFAULT_PORT,
+                             [{pref_public_key_algs, [Alg]},
+                              {user_interaction, false},
+                              {auth_methods, "publickey"},
+                              silently_accept_hosts]),
     {ok, Channel} =
-	ssh_connection:session_channel(ConnectionRef, infinity),
+        ssh_connection:session_channel(ConnectionRef, infinity),
     ok = ssh_connection:close(ConnectionRef, Channel),
     ok = ssh:close(ConnectionRef).
 
 %%--------------------------------------------------------------------
 erlang_server_openssh_client_public_key_dsa() ->
-    [{timetrap, {seconds,(?TIMEOUT div 1000)+10}},
-     {doc, "Validate using dsa publickey."}].
+    [{timetrap, {seconds,(?TIMEOUT div 1000)+10}}].
 erlang_server_openssh_client_public_key_dsa(Config) when is_list(Config) ->
-    erlang_server_openssh_client_public_key_X(Config, ssh_dsa).
+    erlang_server_openssh_client_public_key_X(Config, 'ssh-dss').
 
-erlang_server_openssh_client_public_key_rsa() ->
-    [{timetrap, {seconds,(?TIMEOUT div 1000)+10}},
-     {doc, "Validate using rsa publickey."}].
+erlang_server_openssh_client_public_key_rsa() -> 
+    [{timetrap, {seconds,(?TIMEOUT div 1000)+10}}].
 erlang_server_openssh_client_public_key_rsa(Config) when is_list(Config) ->
-    erlang_server_openssh_client_public_key_X(Config, ssh_rsa).
+    erlang_server_openssh_client_public_key_X(Config, 'ssh-rsa').
 
 
-erlang_server_openssh_client_public_key_X(Config, PubKeyAlg) ->
+erlang_server_openssh_client_public_key_X(Config, Alg) ->
     SystemDir = proplists:get_value(data_dir, Config),
     PrivDir = proplists:get_value(priv_dir, Config),
     KnownHosts = filename:join(PrivDir, "known_hosts"),
     {Pid, Host, Port} = ssh_test_lib:daemon([{system_dir, SystemDir},
-					     {public_key_alg, PubKeyAlg},
+                                             {preferred_algorithms,[{public_key, [Alg]}]},
+                                             {auth_methods, "publickey"},
 					     {failfun, fun ssh_test_lib:failfun/2}]),
-
     ct:sleep(500),
 
-    Cmd = "ssh -p " ++ integer_to_list(Port) ++
-	" -o UserKnownHostsFile=" ++ KnownHosts ++
-	" " ++ Host ++ " 1+1.",
+    Cmd = ssh_test_lib:open_sshc_cmd(Host, Port,
+                                     [" -o UserKnownHostsFile=", KnownHosts,
+                                      " -o StrictHostKeyChecking=no"],
+                                     "1+1."),
     OpenSsh = ssh_test_lib:open_port({spawn, Cmd}),
     ssh_test_lib:rcv_expected({data,<<"2\n">>}, OpenSsh, ?TIMEOUT),
     ssh:stop_daemon(Pid).
@@ -396,14 +377,13 @@ erlang_server_openssh_client_public_key_X(Config, PubKeyAlg) ->
 %%--------------------------------------------------------------------
 %% Test that the Erlang/OTP server can renegotiate with openSSH
 erlang_server_openssh_client_renegotiate(Config) ->
-    PubKeyAlg = ssh_rsa,
+    _PubKeyAlg = ssh_rsa,
     SystemDir = proplists:get_value(data_dir, Config),
     PrivDir = proplists:get_value(priv_dir, Config),
     KnownHosts = filename:join(PrivDir, "known_hosts"),
 
     {Pid, Host, Port} = ssh_test_lib:daemon([{system_dir, SystemDir},
-					     {public_key_alg, PubKeyAlg},
-					     {failfun, fun ssh_test_lib:failfun/2}]),
+                                             {failfun, fun ssh_test_lib:failfun/2}]),
     ct:sleep(500),
 
     RenegLimitK = 3,
@@ -411,11 +391,13 @@ erlang_server_openssh_client_renegotiate(Config) ->
     Data =  lists:duplicate(trunc(1.1*RenegLimitK*1024), $a),
     ok = file:write_file(DataFile, Data),
 
-    Cmd = "ssh -p " ++ integer_to_list(Port) ++
-	" -o UserKnownHostsFile=" ++ KnownHosts ++
-	" -o RekeyLimit=" ++ integer_to_list(RenegLimitK) ++"K" ++
-	" " ++ Host ++ " < " ++ DataFile,
-    OpenSsh = ssh_test_lib:open_port({spawn, Cmd}),
+    Cmd = ssh_test_lib:open_sshc_cmd(Host, Port,
+                                     [" -o UserKnownHostsFile=", KnownHosts,
+                                      " -o StrictHostKeyChecking=no",
+                                      " -o RekeyLimit=",integer_to_list(RenegLimitK),"K"]),
+
+
+    OpenSsh = ssh_test_lib:open_port({spawn, Cmd++" < "++DataFile}),
 
     Expect = fun({data,R}) -> 
 		     try
@@ -442,7 +424,7 @@ erlang_server_openssh_client_renegotiate(Config) ->
 	ssh_test_lib:rcv_expected(Expect, OpenSsh, ?TIMEOUT)
     of
 	_ ->
-	    %% Unfortunatly we can't check that there has been a renegotiation, just trust OpenSSH.
+	    %% Unfortunately we can't check that there has been a renegotiation, just trust OpenSSH.
 	    ssh:stop_daemon(Pid)
     catch
 	throw:{skip,R} -> {skip,R}
@@ -451,7 +433,6 @@ erlang_server_openssh_client_renegotiate(Config) ->
 %%--------------------------------------------------------------------
 erlang_client_openssh_server_renegotiate(_Config) ->
     process_flag(trap_exit, true),
-
     IO = ssh_test_lib:start_io_server(),
     Ref = make_ref(),
     Parent = self(),
@@ -464,6 +445,7 @@ erlang_client_openssh_server_renegotiate(_Config) ->
 			     {silently_accept_hosts,true}],
 		  group_leader(IO, self()),
 		  {ok, ConnRef} = ssh:connect(Host, ?SSH_DEFAULT_PORT, Options),
+                  ct:log("Parent = ~p, IO = ~p, Shell = ~p, ConnRef = ~p~n",[Parent, IO, self(), ConnRef]),
 		  case ssh_connection:session_channel(ConnRef, infinity) of
 		      {ok,ChannelId}  ->
 			  success = ssh_connection:ptty_alloc(ConnRef, ChannelId, []),
@@ -486,11 +468,11 @@ erlang_client_openssh_server_renegotiate(_Config) ->
 	    ct:fail("Error=~p",[Error]);
 	{ok, Ref, ConnectionRef} ->
 	    IO ! {input, self(), "echo Hej1\n"},
-	    receive_data("Hej1"),
+	    receive_data("Hej1", ConnectionRef),
 	    Kex1 = ssh_test_lib:get_kex_init(ConnectionRef),
 	    ssh_connection_handler:renegotiate(ConnectionRef),
 	    IO ! {input, self(), "echo Hej2\n"},
-	    receive_data("Hej2"),
+	    receive_data("Hej2", ConnectionRef),
 	    Kex2 = ssh_test_lib:get_kex_init(ConnectionRef),
 	    IO ! {input, self(), "exit\n"},
 	    receive_logout(),
@@ -553,23 +535,29 @@ erlang_client_openssh_server_nonexistent_subsystem(Config) when is_list(Config)
 %%--------------------------------------------------------------------
 %%% Internal functions -----------------------------------------------
 %%--------------------------------------------------------------------
-receive_data(Data) ->
+receive_data(Data, Conn) ->
     receive
 	Info when is_binary(Info) ->
 	    Lines = string:tokens(binary_to_list(Info), "\r\n "),
 	    case lists:member(Data, Lines) of
 		true ->
-		    ct:log("Expected result found in lines: ~p~n", [Lines]),
+		    ct:log("Expected result ~p found in lines: ~p~n", [Data,Lines]),
 		    ok;
 		false ->
 		    ct:log("Extra info: ~p~n", [Info]),
-		    receive_data(Data)
+		    receive_data(Data, Conn)
 	    end;
 	Other ->
 	    ct:log("Unexpected: ~p",[Other]),
-	    receive_data(Data)
-    after 
-	30000 -> ct:fail("timeout ~p:~p",[?MODULE,?LINE])
+	    receive_data(Data, Conn)
+    after
+	30000 ->
+             {State, _} = case Conn of
+                              undefined -> {'??','??'};
+                              _ -> sys:get_state(Conn)
+                          end,
+            ct:log("timeout ~p:~p~nExpect ~p~nState = ~p",[?MODULE,?LINE,Data,State]),
+            ct:fail("timeout ~p:~p",[?MODULE,?LINE])
     end.	
 
 receive_logout() ->
