@@ -53,28 +53,36 @@ all() ->
      {group, 'tlsv1.2'},
      {group, 'tlsv1.1'},
      {group, 'tlsv1'},
-     {group, 'sslv3'}
+     {group, 'sslv3'},
+     {group, 'dtlsv1.2'},
+     {group, 'dtlsv1'}
     ].
 
 groups() ->
-    [{'tlsv1.2', [], packet_tests()},
-     {'tlsv1.1', [], packet_tests()},
-     {'tlsv1', [], packet_tests()},
-     {'sslv3', [], packet_tests()}
+    [{'tlsv1.2', [], socket_packet_tests() ++ protocol_packet_tests()},
+     {'tlsv1.1', [], socket_packet_tests() ++ protocol_packet_tests()},
+     {'tlsv1', [], socket_packet_tests() ++ protocol_packet_tests()},
+     {'sslv3', [], socket_packet_tests() ++ protocol_packet_tests()},
+     %% We will not support any packet types if the transport is
+     %% not reliable. We might support it for DTLS over SCTP in the future 
+     {'dtlsv1.2', [], [reject_packet_opt]},
+     {'dtlsv1', [],  [reject_packet_opt]}
     ].
 
-packet_tests() ->
-    active_packet_tests() ++ active_once_packet_tests() ++ passive_packet_tests() ++
-	[packet_send_to_large,
-	 packet_cdr_decode, packet_cdr_decode_list,
+socket_packet_tests() ->
+    socket_active_packet_tests() ++ socket_active_once_packet_tests() ++ 
+        socket_passive_packet_tests() ++ [packet_send_to_large, packet_tpkt_decode, packet_tpkt_decode_list].
+
+protocol_packet_tests() ->
+    protocol_active_packet_tests() ++ protocol_active_once_packet_tests() ++ protocol_passive_packet_tests() ++
+	[packet_cdr_decode, packet_cdr_decode_list,
 	 packet_http_decode, packet_http_decode_list,
 	 packet_http_bin_decode_multi,
 	 packet_line_decode, packet_line_decode_list,
 	 packet_asn1_decode, packet_asn1_decode_list,
-	 packet_tpkt_decode, packet_tpkt_decode_list,
 	 packet_sunrm_decode, packet_sunrm_decode_list].
 
-passive_packet_tests() ->
+socket_passive_packet_tests() ->
     [packet_raw_passive_many_small,
      packet_0_passive_many_small,
      packet_1_passive_many_small,
@@ -85,12 +93,8 @@ passive_packet_tests() ->
      packet_1_passive_some_big,
      packet_2_passive_some_big,
      packet_4_passive_some_big,
-     packet_httph_passive,
-     packet_httph_bin_passive,
-     packet_http_error_passive,
      packet_wait_passive,
      packet_size_passive,
-     packet_baddata_passive,
      %% inet header option should be deprecated!
      header_decode_one_byte_passive,
      header_decode_two_bytes_passive,
@@ -98,7 +102,14 @@ passive_packet_tests() ->
      header_decode_two_bytes_one_sent_passive
     ].
 
-active_once_packet_tests() ->
+protocol_passive_packet_tests() ->
+    [packet_httph_passive,
+     packet_httph_bin_passive,
+     packet_http_error_passive,
+     packet_baddata_passive
+    ].
+
+socket_active_once_packet_tests() ->
     [packet_raw_active_once_many_small,
      packet_0_active_once_many_small,
      packet_1_active_once_many_small,
@@ -108,12 +119,16 @@ active_once_packet_tests() ->
      packet_0_active_once_some_big,
      packet_1_active_once_some_big,
      packet_2_active_once_some_big,
-     packet_4_active_once_some_big,
+     packet_4_active_once_some_big
+    ].
+
+protocol_active_once_packet_tests() ->
+    [
      packet_httph_active_once,
      packet_httph_bin_active_once
     ].
 
-active_packet_tests() ->
+socket_active_packet_tests() ->
     [packet_raw_active_many_small,
      packet_0_active_many_small,
      packet_1_active_many_small,
@@ -124,10 +139,7 @@ active_packet_tests() ->
      packet_1_active_some_big,
      packet_2_active_some_big,
      packet_4_active_some_big,
-     packet_httph_active,
-     packet_httph_bin_active,
      packet_wait_active,
-     packet_baddata_active,
      packet_size_active,
      %% inet header option should be deprecated!
      header_decode_one_byte_active,
@@ -136,6 +148,13 @@ active_packet_tests() ->
      header_decode_two_bytes_one_sent_active
     ].
 
+
+protocol_active_packet_tests() ->
+    [packet_httph_active,
+     packet_httph_bin_active,
+     packet_baddata_active
+    ].
+
 init_per_suite(Config) ->
     catch crypto:stop(),
     try crypto:start() of
@@ -168,8 +187,13 @@ init_per_group(GroupName, Config) ->
     end.
 
 
-end_per_group(_GroupName, Config) ->
-    Config.
+end_per_group(GroupName, Config) ->
+    case ssl_test_lib:is_tls_version(GroupName) of
+        true ->
+            ssl_test_lib:clean_tls_version(Config);
+        false ->
+            Config
+    end.
 
 init_per_testcase(_TestCase, Config) ->
     ct:timetrap({seconds, ?BASE_TIMEOUT_SECONDS}),
@@ -1901,6 +1925,25 @@ header_decode_two_bytes_one_sent_passive(Config) when is_list(Config) ->
     ssl_test_lib:close(Server),
     ssl_test_lib:close(Client).
 
+%%--------------------------------------------------------------------
+reject_packet_opt() ->
+    [{doc,"Test packet option is rejected for DTLS over udp"}].
+
+reject_packet_opt(Config) when is_list(Config) ->
+
+    ServerOpts = ssl_test_lib:ssl_options(server_opts, Config),
+       
+    {error,{options,{not_supported,{packet,4}}}} = 
+        ssl:listen(9999, [{packet, 4} | ServerOpts]),
+    {error,{options,{not_supported,{packet_size,1}}}} =  
+        ssl:listen(9999, [{packet_size, 1} | ServerOpts]),
+    {error,{options,{not_supported,{header,1}}}} =
+        ssl:listen(9999, [{header, 1} | ServerOpts]),
+    
+    client_reject_packet_opt(Config, {packet,4}),
+    client_reject_packet_opt(Config, {packet_size, 1}),
+    client_reject_packet_opt(Config, {header, 1}).
+
 %%--------------------------------------------------------------------
 %% Internal functions ------------------------------------------------
 %%--------------------------------------------------------------------
@@ -2223,3 +2266,23 @@ add_tpkt_header(IOList) when is_list(IOList) ->
     Binary = list_to_binary(IOList),
     L = size(Binary) + 4,
     [3, 0, ((L) bsr 8) band 16#ff, (L) band 16#ff , Binary].
+
+
+client_reject_packet_opt(Config, PacketOpt) ->
+    ServerOpts = ssl_test_lib:ssl_options(server_opts, Config),
+    ClientOpts = ssl_test_lib:ssl_options(client_opts, Config),
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+
+    Server = ssl_test_lib:start_server([{node, ClientNode}, {port, 0},
+                                        {from, self()},
+                                        {mfa, {ssl_test_lib, no_result_msg ,[]}},
+                                        {options, ServerOpts}]),
+    Port = ssl_test_lib:inet_port(Server),
+    Client = ssl_test_lib:start_client_error([{node, ServerNode}, {port, Port},
+                                              {host, Hostname},
+                                              {from, self()},
+                                              {mfa, {ssl_test_lib, no_result_msg, []}},
+                                              {options, [PacketOpt |
+                                                         ClientOpts]}]),
+    
+    ssl_test_lib:check_result(Client, {error, {options, {not_supported, PacketOpt}}}).
