@@ -162,7 +162,7 @@ tty_log_open(Log) ->
 		{ok,D} -> D;
 		_ -> unlimited
 	    end,
-    error_logger:add_report_handler(?MODULE, {Fd,Depth}),
+    error_logger:add_report_handler(?MODULE, {Fd,Depth,latin1}),
     Fd.
 
 tty_log_close() ->
@@ -393,11 +393,11 @@ dl_format_1([], [], _, Facc, Aacc) ->
 %%% calling error_logger_tty_h:write_event/2.
 %%%
 
-init({_,_}=St) ->
+init({_,_,_}=St) ->
     {ok,St}.
 
-handle_event(Event, {Fd,Depth}=St) ->
-    case error_logger_tty_h:write_event(tag_event(Event), io_lib, Depth) of
+handle_event(Event, {Fd,Depth,Enc}=St) ->
+    case error_logger_tty_h:write_event(tag_event(Event), io_lib, {Depth,Enc}) of
 	ok ->
 	    ok;
 	Str when is_list(Str) ->
@@ -405,7 +405,7 @@ handle_event(Event, {Fd,Depth}=St) ->
     end,
     {ok,St}.
 
-terminate(_Reason, {Fd,_}) ->
+terminate(_Reason, {Fd,_,_}) ->
     ok = file:close(Fd),
     [].
 
